/*
 * common.c  -  General routines common to all controllers. 
 *
 * Author: John DiMarco, University of Toronto, CDF
 *         jdd@cdf.toronto.edu
 * 
 */

#ifndef lint
static char rcsid[] = "$Id: common.c,v 1.10 2000/01/25 13:58:53 jdd Exp $";
#endif

#include <stdio.h>
#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <nlist.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include <errno.h>
#include "utils.h"
#include "uscsi.h"
#include "format.h"
#include "common.h"

#define BUFFSIZE 255

#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

/* include OS-dependent stuff */
#ifdef SYSV
#include "sunos5.h"
#else
#include "sunos4.h"
#endif

char *kvmstr(address)
u_long address;
{
	static char buff[BUFFSIZE];
	char *bp;

	/* 
	 * Get one byte at a time until buffsize or null byte.  Otherwise
	 * this can trigger a bug in the kernel and cause a system panic.
	 */
	for(bp=buff;bp<(buff+BUFFSIZE);bp++){
		if(-1==kvm_read(kd, address+(bp-buff), bp, sizeof(char))){
			Error("kvmstr");
		}
		if('\0'==*bp) break;
	}
	buff[BUFFSIZE-1]='\0'; /* just in case it's not null-terminated */
	return(buff);
}
	
void boot_synch_message(ctlr_type, ctlr_num, i, speed, width)
char *ctlr_type;	/* type of controller */
int ctlr_num;  	 	/* controller number */
int width;     	 	/* scsi transfer width in bits */ 
int i;  		/* target index */
unsigned int speed;	/* target speed */
{
	switch(speed) {
		case ((u_int)-1) :
			printf(
	  "%s%d:\tTarget %d now Synchronous at unknown max transmit rate\n",
			ctlr_type, ctlr_num, i);
			break;
		case 0 :
			break;
		default :
			printf(
	"%s%d:\tTarget %d now Synchronous at %d.%.3d mb/s max transmit rate\n",
			 ctlr_type, ctlr_num, i,
				(width/8)*speed/1000, (width/8)*speed%1000);
	}
}

void do_scsi_probe(device, devicename, lun)
char *device, *devicename;
int lun;
{
#ifdef SKIP_EPOCH
				/* Do not attempt to probe any Epoch/EMC     *
				 * library-unit devices (they can hard-hang).*/
				if (STREQ("ss", device) ||
				    STREQ("mo", device) ||
				    STREQ("sj", device) ||
				    STREQ("ds", device)) 
				       return;
#endif /* SKIP_EPOCH */
	printf("\t");
	if(-1==(scsi_probe(devicename, lun))){
		switch(errno){
		case EACCES:
			printf(
"<<< SCSI probe failed: no permission to access device special file >>>\n");
			break;
		case ENOTTY:
			printf(
"<<< SCSI probe failed: device driver doesn't support probing >>>\n");
			break;
		case ENXIO:
		case EIO:
			printf(
"<<< SCSI probe failed: device not accessible (media not loaded?) >>>\n");
			break;
		case EBUSY:
			printf(
"<<< SCSI probe failed: device is busy >>>\n");
			break;
		case ENOENT:
			printf(
"<<< SCSI probe failed: device special file not available >>>\n");
			break;
		default:
			perror("scsi probe failed");
			break;
		}
	} else {
		printf("\n");
	}
}

char *compute_devicename(dname, devicepath, addrname)
char *dname, *devicepath, *addrname;
{
	char *devicename;

	/* compute device name */
	if(STREQ(SUNDISK, dname) || STREQ(SSADISK, dname)){
		/* 
		 * This is a disk. 
		 *
		 * We want to query the device corresponding to partition c/2 
		 * eg. rsd0c or sd@0,0:c,raw
		 */
		devicename = cat(
			devicepath, "/", 
			DEV_PREFIX, dname, 
			DEV_INFIX, addrname, 
			DISK_SUFFIX, NULL);
	} else if(STREQ(SUNTAPE, dname)) {
		/* 
		 * This is a tape.
		 *
		 * We want to query the no-rewind BSD-style device.
		 * eg. nrst0 or st@0,0:bn
		 */
		devicename = cat(
			devicepath, "/",
			TAPE_PREFIX,
			dname, DEV_INFIX,
			addrname,
			TAPE_SUFFIX, NULL);
	} else if(STREQ(SUNSST, dname)) {
		/*
		 * This is Sun's simple SCSI target driver, which is used by 
		 * various folk for miscellaneous devices.
		 * 
		 * We want to query the character device,
		 * e.g. rsst0 or sst@0,0:character
		 */
		devicename = cat(
			devicepath, "/", 
			DEV_PREFIX,
			dname, DEV_INFIX, 
			addrname, 
			DEV_SUFFIX, SST_SUFFIX, NULL);
	} else {
		/* 
		 * Don't know what device this is. 
		 * 
		 * Use best guess for devicename: eg. rNN0 or NN@0,0:
		 */ 
		devicename = cat(
			devicepath, "/", 
			DEV_PREFIX,
			dname, DEV_INFIX, 
			addrname, 
			DEV_SUFFIX, NULL);
	}
	return(devicename);
}
