/*
 * format.c - code for providing format info, generating format.dat using
 *            uscsi queries.
 *
 * Author: John DiMarco,
 *         University of Toronto, Computing Disciplines Facility
 *         jdd@cdf.toronto.edu
 *
 */

#ifndef lint
static char rcsid[] = "$Id: format.c,v 3.30 2000/02/24 21:38:52 jdd Exp $";
#endif

#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "utils.h"
#include "uscsi.h"
#include "param.h"

#ifndef DTYPE_DIRECT
#define DTYPE_DIRECT 0x00
#endif /* DTYPE_DIRECT */

#ifndef DTYPE_SEQUENTIAL
#define DTYPE_SEQUENTIAL 0x01
#endif /* DTYPE_SEQUENTIAL */

#define MAX_RPM 32767
#define MIN_RPM 3000
#define STANDARD_ACYL 2
#define FUDGEFACTOR 1.15
#define SECTORSIZE 512
#define MD21 "MD21"
#define SCSI "SCSI"

extern int d;

static int acyl = STANDARD_ACYL;

static void basereport(devicetype, vendor, product, revision, serial, cap)
int devicetype, cap;
char *vendor, *product, *revision, *serial;
{


	printf("Vendor:                      %s\n", vendor);
	printf("Model:                       %s\n", product);
	printf("Revision:                    %s\n", revision);
	printf("Serial:                      %s\n", serial);
	printf("Device Type:                 %s\n", scsi_dtype(devicetype));
	printf("Data Transfer Width(s):      ");
		if(cap&INQ_WBUS32)	printf("32, ");
		if(cap&INQ_WBUS16)	printf("16, ");
		printf("8\n");
#ifdef LONG_REPORT_FORMAT
#define SUPPORTS(c, f) (((c)&(f))?"Supported":"Unsupported")
	printf("Relative Addressing:         %s\n", SUPPORTS(cap,INQ_RELADR));	
	printf("Synchronous Transfers:	     %s\n", SUPPORTS(cap,INQ_SYNC));
	printf("Linked Commands:             %s\n", SUPPORTS(cap,INQ_LINKED));
	printf("Tagged Queuing:	             %s\n", SUPPORTS(cap,INQ_CMDQUE));
	printf("Soft Reset:                  %s\n", SUPPORTS(cap,INQ_SFTRE));
#else /* !LONG_REPORT_FORMAT */
	printf("Supported Features:         ");
	if(cap&INQ_RELADR) printf(" %s", INQ_TXT_RELADR);
	if(cap&INQ_SYNC)   printf(" %s", INQ_TXT_SYNC);
	if(cap&INQ_LINKED) printf(" %s", INQ_TXT_LINKED);
	if(cap&INQ_CMDQUE) printf(" %s", INQ_TXT_CMDQUE);
	if(cap&INQ_SFTRE)  printf(" %s", INQ_TXT_SFTRE);
	printf("\n");
#endif /* LONG_REPORT_FORMAT */
}

static void format(formatdat, osversion, fd, diskdevice, 
			original, vendor, product, revision, serial, cap)
int formatdat, fd;
char *osversion, *diskdevice, *vendor, *product, *revision, *serial;
int original, cap;
{
        unsigned long capacity;
        int nsect, sector_size, cache_status;
        int pcyl, nhead, rpm=-1;
        static int trks_zone, atrks, asect;/* 4.1.1 cc bad code unless static */
        char disk_type[VENDOR_SIZE+PRODUCT_SIZE+2], *ctlr, *cname;
        int fmt_time, ncyl, bpt_, csize;

	/* get number of blocks available */
	if(0>scsi_capacity(fd, &capacity)){
		perror(diskdevice);
		Warning("SCSI READ_CAPACITY failed");
		return;
	}

	/* get nsect, sector_size, pcyl, nhead, rpm, disk_type, ctlr, 
	   fmt_time, acyl, ncyl, bpt, trks_zone, atrks, asect */

	if(0>scsi_modeinfo(fd, original, &nsect, &sector_size, &pcyl, &nhead, 
		&rpm, &trks_zone, &asect, &atrks, &cache_status)){
		perror(diskdevice);
		Warning("SCSI mode select failed.");
	}
	if(SECTORSIZE != sector_size){
		if(sector_size){
			Warning(
		"Drive is formatted with with sector size %d, should be %d",
				sector_size, SECTORSIZE);
			return;
		} else {
			Warning("Drive claims sector size 0, using %d bytes",
				SECTORSIZE);
			sector_size=SECTORSIZE;
		}
	}
	if(-1==rpm){
		/* Warning("Drive doesn't report rpm.") */ ;
	} else if(rpm>MAX_RPM || rpm<MIN_RPM){
		Warning( "Drive claims to have strange RPM value: %d", rpm);
		rpm=-1;
	}
	if(nhead<1) {
		Warning(
	"Drive claims to have %d heads. This makes no sense. Giving up.", 
		nhead);
		return;
	}

	if(nsect<1) {
		Warning(
	"Drive claims to have %d sectors. This makes no sense. Giving up.", 
		nsect);
		return;
	}

	sprintf(disk_type, "%s %s", vendor, product);

	{
		int osmajor=0, osminor=0, subosminor=0;

		sscanf(osversion, "%d.%d.%d", &osmajor, &osminor, &subosminor);
		if(4==osmajor && osminor<2 && subosminor<2){
			ctlr=MD21;
		} else {
			ctlr=SCSI;
		}
		if(osmajor<5) {
			cname="c";
		} else {
			cname="2";
		}
	}

	fmt_time = (int)(capacity / 1000000) + 1;

	ncyl = (capacity / (nhead * nsect)) - acyl;

	bpt_ = (int) (FUDGEFACTOR * SECTORSIZE) * nsect;

	csize = ncyl*nhead*nsect;

	if(formatdat){
		printf("disk_type = \"%s\" \\\n", disk_type);
		printf(  "\t: ctlr = %s : fmt_time = %d \\\n", ctlr, fmt_time);

		/* only output the following if they're nondefault */
		printf(  "\t");
		if(1 != trks_zone) printf(": trks_zone = %d ", trks_zone);
		if(nhead != atrks) printf(": atrks = %d ", atrks);
		if(1 != asect) printf(" : asect = %d ", asect);
		printf(" \\\n");

		printf(  "\t: ncyl = %d : acyl = %d", ncyl, acyl);
		printf(     " : pcyl = %d : nhead = %d", pcyl, nhead);
		printf(     " : nsect = %d \\\n", nsect);

		printf(  "\t: rpm = %d : bpt = %d\n", (-1==rpm)?3600:rpm, bpt_);

		printf("\n");
		printf("partition = \"%s\" \\\n", disk_type);
		printf(  "\t: disk = \"%s\" : ctlr = %s \\\n", disk_type, ctlr);
		printf(  "\t: %s = 0, %d\n", cname, csize);
	} else {
		basereport(DTYPE_DIRECT,vendor,product,revision,serial,cap);
		printf("Formatted Capacity:          %ld sectors ", capacity); 
		if(capacity >= 2000000){
			/* 
			 * I want GB to the first decimal place, since that
			 * is what people are used to when they buy disks. 
			 */
			printf("(%.1f GB)\n", (((float)capacity)/2000000.0));
		} else {
			/* 
			 * I really only care about MB to the nearest 
			 * integer but I'll work in terms of floats just in 
			 * case I change my mind. 
			 */
			printf("(%.0f MB)\n", (((float)capacity)/2000.0));
		}
		printf("Sector size:                 %d bytes\n", sector_size);
		printf("Physical Cylinders:          %d\n", pcyl);
		printf("Heads:                       %d\n", nhead);
		printf("Sectors per track (Avg.):    %d\n", nsect);
		printf("Tracks per zone:             %d\n", trks_zone);
		printf("Alternate Sectors per zone:  %d\n", asect);
		printf("Alternate Tracks per volume: %d\n", atrks);
		if(-1==rpm){
			printf("Rotational speed:            NA\n", rpm);
		} else {
			printf("Rotational speed:            %d rpm\n", rpm);
		}
		printf("Cache:                       ");
		switch(cache_status){
			case 0:
				printf("Disabled\n");
				break;
			case 1:
				printf("Read-Only\n");
				break;
			case 2:
				printf("Write-Only\n");
				break;
			case 3:
				printf("Read-Write\n");
				break;
			default:
				printf("Unknown\n");
				break;
		}
	}
}

void report(formatdat, osversion, device, original)
int formatdat;
char *osversion;
char *device;
int original;
{
	struct stat sb;
	int fd, i, cap;
	u_int devicetype;
	u_int max_bll, min_bll;
	static char vendor[VENDOR_SIZE+1];
	static char product[PRODUCT_SIZE+1];
	static char revision[REVISION_SIZE+1];
	static char serial[SERIAL_SIZE+1];

	/* preliminary stuff */
	if(d) printf("Generating report for %s, SunOS %s\n", device,
			osversion);
	if(stat(device, &sb)<0){
		perror(device);
		Warning("Cannot stat device.");
		return;
	}
	if(!S_ISCHR(sb.st_mode)){
		Warning("%s must be a raw SCSI device.", device);
		return;
	}
	if(param_is_set(P_OPENCHECK) && opencheck(device)){
		Warning("Device is currently busy.");
		return;
	}
	if(0>(fd=open(device, O_RDONLY|O_NDELAY))){
		perror(device);
		Warning("Cannot open device.");
		return;
	}

	/* get vendor, product, revision, and serial */
	strncpy(serial, "Unknown", sizeof(serial));
	if(0>scsi_inquiry(fd, &devicetype, vendor, product, revision, 
			serial, &cap)){
		perror(device);
		Warning("SCSI INQUIRY failed.");
		return;
	}

	/* ensure null termination without trailing spaces */
	vendor[VENDOR_SIZE]='\0';
	for(i=VENDOR_SIZE-1;i>0 && vendor[i]==' ';i--) vendor[i]='\0';
	product[PRODUCT_SIZE]='\0';
	for(i=PRODUCT_SIZE-1;i>0 && product[i]==' ';i--) product[i]='\0';
	revision[REVISION_SIZE]='\0';
	for(i=REVISION_SIZE-1;i>0 && revision[i]==' ';i--) revision[i]='\0';
	serial[SERIAL_SIZE]='\0';
	for(i=SERIAL_SIZE-1;i>0 && serial[i]==' ';i--) serial[i]='\0';

	if(formatdat && DTYPE_DIRECT!=devicetype){
	     Warning("Can't create format.dat entry for device of type: %s\n",
			scsi_dtype(devicetype));
		return;
	}
	switch(devicetype){
	case DTYPE_DIRECT:
		format(formatdat, osversion, fd, device, original, vendor, 
			product, revision, serial, cap);
		break;
	case DTYPE_SEQUENTIAL:
		if(0>scsi_read_blklim(fd, &max_bll, &min_bll)){
			perror(device);
			Warning("SCSI READ BLOCK LIMITS failed.");
			return;
		}
		basereport(devicetype, vendor, product, revision, serial, cap);
		printf("Maximum block length:        %u bytes\n", max_bll);
		printf("Minimum block length:        %u bytes\n", min_bll);
		break;
	default:
		basereport(devicetype, vendor, product, revision, serial, cap);
		break;
	}
}

void banner(formatdat, report, devicecount)
int formatdat, report;
u_int devicecount; /* NOTUSED */
{
	if(devicecount>1){
		if(formatdat){
			printf("\n");
		} else if(report){
			printf("----------------------------\n");
		}
	}
}
