/*
 * glm.c  -  process GLM host adapters.
 *
 * Author: John DiMarco, University of Toronto, CSLab
 *         jdd@cs.toronto.edu
 * 
 */

#ifndef lint
static char rcsid[] = "$Id: glm.c,v 1.11 2000/12/22 14:05:35 jdd Exp $";
#endif

#ifdef GLM_SUPPORT

#include "uscsi.h"
#include "format.h"
#include <stdio.h>

#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <nlist.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include "utils.h"
#include "common.h"
#include "param.h"

#ifdef SYSV
#include "sunos5.h"

#ifdef _NOTE
#undef _NOTE
#endif /* _NOTE */
#define _NOTE(x) 

#include <sys/scsi/adapters/glmreg.h>
#include <sys/scsi/adapters/glmvar.h>


#ifndef CNUM
#define CNUM(glm)      (ddi_get_instance(glm->g_tran->tran_hba_dip))
#endif /* CNUM */

#undef SYNC_KNOWN
#undef NOWIDE
#undef NOISY
#undef NEXT

#ifndef SYNC_KNOWN
#define SYNC_KNOWN(i) ((glm->g_syncstate[i])==NSYNC_SDTR_DONE)
#endif /* SYNC_KNOWN */

#ifndef NOWIDE
#define NOWIDE(i) (glm->g_nowide & (1<<(i)))
#endif /* NOWIDE */

#ifndef NOISY
#define NOISY(i) (glm->g_backoff & (1<<(i)))
#endif /* NOISY */

#ifndef NEXT
#define NEXT(g) ((u_long)(g)->g_next)
#endif /* NEXT */

/* 
 * The following is a sanity check.  We know there are no glms of type 0,
 * so if glm->g_devid is zero, it's very likely that the kernel data 
 * structure we're looking at isn't what we think it is, and hence our 
 * results are bogus.  Similarly, if the clock of the controller isn't 
 * 10Mhz x Power-of-2, it's likely bogus.
 * 
 */
#ifndef INSANE
#define INSANE (0==glm->g_devid) || (	(10!=glm->g_sclock) && \
					(20!=glm->g_sclock) && \
					(40!=glm->g_sclock) && \
					(80!=glm->g_sclock) && \
					(160!=glm->g_sclock) && \
					(320!=glm->g_sclock) )
#endif /* INSANE */

static char *glmType(typecode)
unsigned int typecode;
{
	/* see /usr/include/sys/scsi/adapters/glmreg.h */
	switch(typecode){
	case GLM_53c825:		return("NCR53C825");
	case GLM_53c875:		return("NCR53C875");
	default:	return("unknown");
	}
}

static u_int glm_get_speed(glm, i)
struct glm *glm;
int i; /* target index */
{
	u_int speed;

	/*
	 * Synch cycle time in milliseconds is the
	 * number of ticks in the period divided
	 * by the tick rate. glm_sclock is the
	 * glm's tick rate in units of 1MHz.
	 */

	/* 
         * We don't seem to need to worry about the clock anymore; the
	 * period seems to indicate the number of nanoseconds per xfer 
	 * (where xfer is one byte for narrow, two bytes for wide) 
         */

	speed = glm->g_minperiod[i]?(1000000/glm->g_minperiod[i]):0;

	if(!NOWIDE(i)) speed *= 2;

#ifdef DEBUG
	dfprintf(2, stderr, "Target %d: sclock %u, minperiod %u, nowide %u\n",
			i, glm->g_sclock, glm->g_minperiod[i], NOWIDE(i));
#endif /* DEBUG */

	return(speed);
}

static void glm_controller_message(cnum, glm)
int cnum;
struct glm *glm;
{
	printf("glm%d:", cnum);
	printf(" type %d (%s)", (u_int)glm->g_devid,
		glmType(glm->g_devid));
	printf(" speed %dMHz", (u_int)glm->g_sclock);
	printf("\n");
}

static void glm_synch_message(glm, i, speed)
struct glm *glm;
int i;
u_int speed;
{
	if(speed && SYNC_KNOWN(i)){
		printf("Synchronous(%d.%.3dMB/sec)", speed/1000, speed%1000);
	} else {
		printf("Asynchronous");
	}
	if(NOISY(i)){
		printf(" Noisy");
	} else {
		printf(" Clean");
	}
#ifdef TAGGED
        if(TAGGED(i)){
                printf(" TaggedQueuing");
        } else {
                printf(" NoTaggedQueuing");
        }
#endif /* TAGGED */
	if(NOWIDE(i)){
		printf(" Narrow");
	} else {
		printf(" Wide");
	}
}

int process_glm()
{
	u_long addr;
	struct glm glm[1];
	int noisycount=0; /* number of noisy devices */
	u_int j, nglm;

        if(-1==kvm(nl[N_GLM_HEAD].n_value, addr)){
		return(0); /* glm driver not loaded */
        }
        if((u_long)NULL==addr) return(0); /* no glm's */

        if(-1==kvm(addr, glm[0])){
                Error("kvm read: initial glm");
        }

	nglm=get_nglm();
	if(0==nglm) return(0);

	for(j=0;j<nglm;j++){
		struct dev_info di, *next;
		char *devicepath, *dname;
		scsi_hba_tran_t tran;

		if((u_long)NULL==addr) break;

                if(-1==kvm(addr, glm[0])){
			Error("kvm read: glm #%d", j);
		}

		/* 
		 * Grab the transport devinfo structure for the glm
		 * controller. Needed for CNUM. 
		 */
		if(-1==kvm((u_long)glm->g_tran, tran)){
			Error("kvm read: #%d glm tran", j);
		}
		glm->g_tran=(scsi_hba_tran_t *)&tran; 
		if(-1==kvm((u_long)glm->g_tran->tran_hba_dip, devi)){
			Error("kvm read: #%d glm devinfo", j);
		}
		glm->g_tran->tran_hba_dip=(dev_info_t *)&devi;

		/* 
		 * grab the devinfo structure for the glm controller. CNUM 
		 * requires it, and we need it to determine the device names.
		 */
		if(-1==kvm(glm->g_dip, devi)){
			Error("kvm read: #%d glm devinfo", j);
		} 

		devicepath = get_devicepath(j);

		glm->g_dip=(dev_info_t)&devi; 

#ifdef INSANE
		if(INSANE){
			Warning(
"The kernel data structure for glm%d failed a sanity check.\n\
  There may be a mismatch between the compile-time headers and the currently\n\
  running kernel.  A recompile on this machine may be necessary.", CNUM(glm));
		}
#endif /* INSANE */

		if(param_is_set(P_CONTROLLER)) {
			glm_controller_message(CNUM(glm), glm);
		}

		init_scsitarget();

		/* 
		 * Now, traverse glm's devinfo child list, looking for
		 * attached drivers (i.e. DDI_CF2 is true). If we find
		 * an attached driver, we presume it is an active device.
		 */

		for(next=ddi_get_child(&devi);
		    NULL!=next; 
		    next=ddi_get_next_sibling(&di)){
			char *devicename, *addrname;
			int target, lun;

			if(-1==kvm(next, di)){
			    Error("kvm read: #%d glm devinfo child", j);
			}

			if(!DDI_CF2(&di)) continue; /* no device attached */

			devicecount++;

			/*
			 * Get the device name and address, and 
			 * compute the device path for it
			 */

			/* get device name */
			dname=s(kvmstr((u_long)ddi_get_name(&di)));
			/* ... and address */
			addrname=s(get_addr(&di));

			devicename = compute_devicename(dname, 
						devicepath, addrname);

			if(-1==get_scsiaddr(dname, &di, &target, &lun)){
				Error("scsiaddr: #%d", j);
			}

			if(0==lun && NOISY(target)) noisycount++;

			banner(param_is_set(P_FORMATDAT),
				param_is_set(P_REPORT), devicecount);
			if(!param_is_set(P_QUIET)){
				if(param_is_set(P_SUNOS4STYLE)) {
					if(lun==0){
						boot_synch_message("glm",
							CNUM(glm), target, 
							glm_get_speed(glm, 
								target), 
								NARROW);
					}
				} else {
					printf("glm%d: %s%s tgt %d lun %d:\n", 
						CNUM(glm), dname, addrname, 
						target, lun);
					if(param_is_set(P_SYNCH)){
						printf("\t");
						glm_synch_message(glm, target, 
						 glm_get_speed(glm, target));
						printf("\n");
					}
				}
			}
			if(param_is_set(P_PROBE)) {
				do_scsi_probe(dname, devicename, lun);
			}
			if(param_is_set(P_REPORT)) {
				report(param_is_set(P_FORMATDAT),
					osrelease, devicename,
					param_is_set(P_ORIGINAL));
			}
			free(devicename);
			free(addrname);
			free(dname);
		}

		free(devicepath);

		addr=NEXT(glm);
	}
	return(noisycount);
}

#else /* !SYSV */
/* GLM isn't supported on SunOS 4.x */
int process_glm(){}
#endif /* SYSV */
#endif /* GLM_SUPPORT */
