
/************************************************************************/
/*																		*/
/* ptisp.h: Header file for ptisp.c										*/
/*																		*/
/* Copyright (c) 1993 by Sun Microsystems, Inc.							*/
/*																		*/
/* From: ispvar.h	1.25	93/07/29 PTI								*/
/*																		*/
/************************************************************************/

static char sccsid_ptisp_h[] = "@(#) ptisp.h 96/12/11 14.1.2.1 96/12/11     Copyright 1993,94 Performance Technologies, Inc."; 

/*
 * Convenient short hand defines
 */

#define	TRUE					1
#define	FALSE					0
#define	UNDEFINED				-1

#define	MSW(x)					(short) (((long) x >> 16) & 0xFFFF)
#define	LSW(x)					(short) ((long) x & 0xFFFF)

#define	NPTISPBUS				8
#ifdef NTARGETS
#undef NTARGETS
#endif
#define	NTARGETS				16

#define	TGT(sp)					((sp)->cmd_pkt.pkt_address.a_target)
#define	LUN(sp)					((sp)->cmd_pkt.pkt_address.a_lun)

#define	PTISP_RETRY_DELAY		5
#define	PTISP_RETRIES			0

/*
 * On-board Status Register offset from register attribute.
 */

#define PTISP_OBSTAT_OFFSET		0x10000
#define	PTISP_OBSTAT_ECOUNT		10		/* report err after 10 in a row	*/

/* 
 * OBSTAT bit defines
 */

#define PTISP_OBSTAT_TERMPWR	0x80	/* Termpwr state				*/
#define PTISP_OBSTAT_FUSERR		0x40	/* Fuse Error (open)			*/
#define PTISP_OBSTAT_PODIS		0x20	/* Power Up Disable State		*/
#define PTISP_OBSTAT_DIFFSNS	0x10	/* Differential Sense			*/
#define PTISP_OBSTAT_INTMSK		0x0c	/* Mask for interrupt level		*/
#define PTISP_OBSTAT_SWMSK		0x03	/* Switch pack mask				*/

/*
 * Interrupt actions returned by ptisp_i_flag_event()
 */

#define	PTISP_ACTION_CONTINUE	0
#define	PTISP_ACTION_RETURN		1
#define	PTISP_ACTION_IGNORE		2

/*
 * Reset actions for ptisp_i_reset_interface()
 */

#define	PTISP_RESET_BUS_IF_BUSY	0x01	/* reset scsi bus if it is busy	*/
#define	PTISP_FORCE_RESET_BUS	0x02	/* reset scsi bus on error reco */

/*
 * timeout values
 */

#define	PTISP_TIMEOUT_GRACE		50		/* Timeout margin (sec.)		*/
#define	PTISP_TIMEOUT_DELAY(s, d)		(s * (1000000 / d))

/*
 * delay time for polling loops
 */

#define	PTISP_POLL_DELAY_TIME	1000	/* usecs						*/

/*
 * busy wait delay time after chip reset
 */

#define	PTISP_RESET_BSY_WAIT_TM	100		/* usecs						*/

/*
 * timeout for ISP coming out of reset
 */

#define	PTISP_RESET_WAIT_TIME	1000	/* ms							*/
#define	PTISP_SOFT_RESET_TIME	1		/* second						*/

/*
 * timeout for reading the response
 */

#define	PTISP_RESPONSE_TIME		100

/*
 * Size definitions for request and response queues.
 */

#define	PTISP_MAX_REQUESTS		256
#define	PTISP_MAX_RESPONSES		256

/*
 * ISP request packet as defined by the Firmware Interface
 */

struct ptisp_dataseg {
	long		d_base;
	long		d_count;
};

/*
 * The request/response header entry
 */

struct ptisp_cq_header {
	char		entry_count;
	char		entry_type;
	u_char		flags;
	u_char		seqno;
};

struct ptisp_request {
	struct ptisp_cq_header header;
	opaque_t	token;
	char		target;
	char		lun_trn;
	union {
		short	cdblen;
		short	modifier;
	} cm;
	short		flags;
	short		reserved;
	short		timeout;
	short		seg_count;
	u_char		cdb[12];
	struct ptisp_dataseg dataseg[4];
};

/*
 * Header flags definitions
 */

#define	PTISP_CQ_FLAG_CONT			0x01
#define	PTISP_CQ_FLAG_FULL			0x02
#define	PTISP_CQ_FLAG_BADHEADER		0x04
#define	PTISP_CQ_FLAG_BADPACKET		0x08
#define	PTISP_CQ_FLAG_ERR_MASK		0x0E

/*
 * Header entry_type definitions
 */

#define	PTISP_CQ_TYPE_REQUEST		1
#define	PTISP_CQ_TYPE_DATASEG		2
#define	PTISP_CQ_TYPE_RESPONSE		3
#define	PTISP_CQ_TYPE_MARKER		4
#define	PTISP_CQ_TYPE_CMDONLY		5

#define	PTISP_CDBMAX				12

/*
 * marker packet (req_modifier) values
 */

#define	PTISP_SYNCHRONIZE_NEXUS		0
#define	PTISP_SYNCHRONIZE_TARGET	1
#define	PTISP_SYNCHRONIZE_ALL		2

/*
 * request flag values
 */

#define	PTISP_REQ_FLAG_NODISCON		0x0001
#define	PTISP_REQ_FLAG_HEAD_TAG		0x0002
#define	PTISP_REQ_FLAG_ORDERED_TAG	0x0004
#define	PTISP_REQ_FLAG_SIMPLE_TAG	0x0008
#define	PTISP_REQ_FLAG_USE_TRN		0x0010
#define	PTISP_REQ_FLAG_DATA_READ	0x0020
#define	PTISP_REQ_FLAG_DATA_WRITE	0x0040
#define	PTISP_REQ_FLAG_MASK			0x00FF

/*
 * throttle values for ISP request queue
 */
#define	PTISP_SHUTDOWN_THROTTLE		-1		/* do not submit any reqs	*/
#define	PTISP_CLEAR_THROTTLE		(PTISP_MAX_REQUESTS - 1)

/*
 * ISP response packet as defined by the Firmware Interface
 */

struct ptisp_response {
	struct ptisp_cq_header header;
	opaque_t	token;
	short		scb;
	short		reason;
	short		state;
	short		flags;
	short		timeout;
	short		rqs_count;
	long		resid;
	long		reserved[2];
	long		sense[8];
};

#define	PTISP_STAT_NEGOTIATE		0x0080

/*
 * ptisp softstate structure
 */

/*
 * This struct is defined for saving time in computing the slot number
 * for a known target.
 */

struct ptisp_slot {
	struct scsi_cmd *cmds[PTISP_MAX_REQUESTS];
};

/*
 * ptisp_pkt_private: Private stuff for a SCSI packet.
 *
 * We use the pkt_ha_private as the pointer to such a struct.  The forw
 * field was in the SCSI packet in Solaris 2.x.  But now we don't have
 * that defined in SunOS 4.1.x.  So this forw has to be moved here.
 */

struct ptisp_pkt_private {
	struct ptisp_response	*rspn;
	struct scsi_cmd			*forw;
	u_short					deadbeef;
	int						tag;
	struct ptisp_pkt_private *next;
	struct ptisp_pkt_private *prev;
};

/*
 * ptisp: Driver unit structure
 *
 * Each host adapter will export the address of this structure,
 * which defines function entry points
 * The library will use the address of this structure to form
 * SCSI device addresses- the address of this structure will
 * be encoded in the 'a_cookie' field of the SCSI device address.
 *
 * All requests are defined such that the SCSI address is either
 * a formal paramter, or contained within a formal parameter.
 *
 * Therefore, each function entry in the driver will know that
 * the 'a_cookie' field points to a transport structure, which
 * will then allow each function to retrieve a pointer to the
 * correct soft_state structure.
 */

struct ptisp {
	struct scsi_transport tran;		/* This one MUST be first			*/

	struct ptisp_slot slots[NTARGETS];

	struct dev_info *dev;			/* Pointer to devinfo structure		*/

	struct ispregs	*reg;			/* ISP1000 register pointer			*/

	struct ptisp_request *rqst_ptr;	/* Current request					*/
	struct ptisp_request *rqst_base;/* Request Q base address			*/
	u_long			rqst_dvma;		/* Request Q DVMA space				*/

	struct ptisp_response *rspn_ptr;/* Current response Q pointer		*/
	struct ptisp_response *rspn_base;
    u_long			rspn_dvma;

	int				qspace;			/* ISP request/response Q pointers	*/

	u_long			burstsize;

	caddr_t			ob_statreg;		/* Ptr to mapped in status register	*/

	u_short			cap[NTARGETS];	/* Target capabilities				*/
	u_short			sync[NTARGETS];

	short			major_rev;		/* Major/minor firmware rev number	*/
	short			minor_rev;

	char			unit_no;		/* Unit number of this card			*/
	char			scsi_id;		/* Host adapter SCSI ID				*/
	u_char			shutdown;		/* HA can't do anything				*/

	u_char			diffmode;		/* Differential mode flag			*/
	u_char			ultramode;		/* Ultra SCSI-3 mode				*/
	u_char			tmoncount;		/* report errors when down to 0		*/
	u_char			display_rate[16];	
									/* Display the rate of drives */
	u_char			intr_level;		/* SBus interrupt level				*/

	struct ptisp_mbox mbox;			/* Mbox values are stored here		*/

	int				prev_tms;		/* TMON interface status			*/
	int				chk_tms;

	u_short			rqst_in;		/* Request/Response in/out counter	*/
	u_short			rqst_out;
	u_short			rspn_in;
	u_short			rspn_out;

	int				ncmds[NTARGETS];/* For ptispinfo only				*/
	int				nreads[NTARGETS];
	int				nwrites[NTARGETS];
	int				kbytes[NTARGETS];
	u_int			sync_told;		/* Bitmap for targets				*/

	u_char			tags[NTARGETS];

	struct buf		rqst_buf;		/* mb_mapalloc buf struct for rqst	*/
	struct buf		rspn_buf;		/* mb_mapalloc buf struct for rspn	*/

	int				npkt_avail;		/* Available private buffers		*/
	int				npkt_used;		/* Used private buffers				*/
	struct ptisp_pkt_private *pkt_head;
	struct ptisp_pkt_private *pkt_free;
	struct ptisp_pkt_private pkts[PTISP_MAX_REQUESTS];

};

/*
 * Handy definitions for the above fields.  They should be used the same
 * as TGT() and LUN() macros because this struct is part of a sp.
 */

#define	PTISP_PKT_PRIVATE(sp) \
	((struct ptisp_pkt_private *) ((sp)->cmd_pkt.pkt_ha_private))

#define	RSPN(sp)	(PTISP_PKT_PRIVATE(sp)->rspn)
#define	FORW(sp)	(PTISP_PKT_PRIVATE(sp)->forw)
#define	TAG(sp)		(PTISP_PKT_PRIVATE(sp)->tag)

/*
 * Cpu types for determine burst sizes
 */

#define	CPU_TYPE_IPC		0x52
#define	CPU_TYPE_IPX		0x57
#define	CPU_TYPE_SPARC2		0x55
#define	CPU_TYPE_SLC		0x54
#define	CPU_TYPE_ELC		0x56
#define	CPU_TYPE_SPARC10	0x72
#define	CPU_TYPE_SPARCMP	0x71
#define	CPU_TYPE_SPARC1P	0x53
#define	CPU_TYPE_SYS5K		0xF7
#define	CPU_TYPE_LX_OR_SPARC5			0x80

/*
 * Speed to SCSI period conversion table.
 */

/* ISP1000 fast wide scsi-2 */
unsigned char ptisp_speed2period_fast[] = {
	0x0, 0x0, 0x0, 0x0, 0x38, 0x32, 0x2d, 0x2a, 0x25, 0x1e, 0x19,
};
char *ptisp_speed2rate_fast[] = {
        "async", "async", "async", "async", "4.5" ,"5", "5.5","6","6.5","8","10"
};

/* ISP1000 ultra wide scsi-3 */
unsigned char ptisp_speed2period[] = {
           0x0,0x42,0x3a,0x32,0x2a,0x21,0x1e,0x19,0x15,0x10,0xc 
};
char *ptisp_speed2rate[11] = {
       "async", "3", "4", "5", "6", "7", "8","10","12","15","20" 
};

/*
 * Firmware download options.  For external variable ptisp_update_fw.
 */

#define	PTISP_FW_UPDATE_OFF	0		/* Don't update firmware	*/
#define	PTISP_FW_UPDATE_NEWER	1		/* Update if firmware is newer	*/
#define	PTISP_FW_UPDATE_ALWAYS	2		/* Force firmware update	*/

/*
 * Default SCSI Host Adapter ID
 */

#define	PTISP_DEFAULT_HOSTID	7

/*
 * Some flags are not defined in SunOS4.1.3.  Fortunately we still have
 * some bits to fake them by ourselves.
 */

#define CFLAG_COMPLETED			0x2000
#define	CFLAG_FINISHED			0x4000
#define	CFLAG_IN_TRANSPORT		0x8000
#define	STAT_BUS_RESET			0x8
#define	STAT_TIMEOUT			0x10
#define	SCSI_OPTIONS_TAG		0x80
#define	SCSI_OPTIONS_WIDE		0x100
#define	STATE_ARQ_DONE			0x20
#define	SCMD_MODE_SENSE_G1		0x5A
#define	SCMD_MODE_SELECT_G1		0x55

/*
 * Unavailable SCSA variables.
 *
 * We need to figure out the initial values for them.
 */

int scsi_tag_age_limit = 100;

/*
 * Our own stuff for onboard monitor.
 */

#ifndef KEND
#define PTISP_SBUS_INT_INSRAM			0xFF00
#define PTISP_SBUS_ID_INSRAM			0xFF80
#else
#define PTISP_SBUS_INT_INSRAM			0xBFFE
#define PTISP_SBUS_ID_INSRAM			0xFFFE
#endif KEND


/* Our additions for SCSI capabilities.  ie case: PTISP_CAP_VALUE + offset */

char *ptisp_scsi_capstrings[] =
{
    "auto-rqsense",
    0
};

#define PTISP_CAP_VALUE		0x8000

#define PTISP_SCSI_CAP_ARQ	0x8000

/*........................... End of ptisp.h ...........................*/
