/*
 * ispvar.h - header file for ISP controller support.
 *
 * Info originally from Solaris 2.4EA
 *
 * $Id: ispvar.h,v 1.3 1997/09/17 20:51:02 jdd Exp $
 */

#ifndef ISP_H
#define ISP_H

/* ISP capability defines */
#define	ISP_CAP_DISCONNECT	0x8000
#define	ISP_CAP_PARITY		0x4000
#define	ISP_CAP_WIDE		0x2000
#define	ISP_CAP_SYNC		0x1000
#define	ISP_CAP_TAG		0x0800
#define	ISP_CAP_AUTOSENSE	0x0400
#define	ISP_CAP_ERRSTOP		0x0200
#define	ISP_CAP_ERRSYNC		0x0100

/* synchronous periods */
#define	ISP_20M_SYNC_PERIOD	0x000C
#define	ISP_10M_SYNC_PERIOD	0x0019
#define	ISP_8M_SYNC_PERIOD	0x0025
#define	ISP_5M_SYNC_PERIOD	0x0032
#define	ISP_4M_SYNC_PERIOD	0x0041

#define	PERIOD_MASK(val)	((val) & 0xff)

#define	CNUM(isp)		(ddi_get_instance(isp->isp_tran.tran_dev))
#define	NTARGETS_WIDE		16
#define	ISP_MAX_REQUESTS	256
#define	ISP_MAX_SLOTS		(NTARGETS_WIDE * ISP_MAX_REQUESTS)
#define	ISP_MAX_MBOX_REGS		6

struct isp_mbox_cmd {
	u_int		timeout;
	u_char		retry_cnt;
	u_char		n_mbox_out;
	u_char		n_mbox_in;
	u_short		mbox_out [ISP_MAX_MBOX_REGS];
	u_short		mbox_in  [ISP_MAX_MBOX_REGS];
};

struct isp_mbox {
	ksema_t			mbox_sema;
	u_char			mbox_flags;
	struct isp_mbox_cmd 	mbox_cmd;
};

struct isp_slot	{
	struct scsi_cmd	*slot_cmd;
	clock_t		slot_deadline;
};

struct isp {
	struct scsi_transport	isp_tran;
#define	isp_dip			isp_tran.tran_dev
#define	isp_iblock		isp_tran.tran_lkarg
	struct isp		*isp_next;
	short			isp_major_rev;
	short			isp_minor_rev;
	u_char			isp_burstsize;
	u_char			isp_initiator_id;
	u_short			isp_cap[NTARGETS_WIDE];
	u_short			isp_synch[NTARGETS_WIDE];
	volatile struct ispregs		*isp_reg;
	struct isp_mbox		isp_mbox;
	u_char			isp_shutdown;
	caddr_t			isp_cmdarea;
	ddi_dma_cookie_t	isp_dmacookie;
	ddi_dma_handle_t	isp_dmahandle;
	u_long			isp_request_dvma,
				isp_response_dvma;
	int			isp_queue_space;
	kmutex_t		isp_request_mutex;
	kmutex_t		isp_response_mutex;
	u_short			isp_request_in,
				isp_request_out;
	u_short			isp_response_in,
				isp_response_out;
	struct isp_request	*isp_request_ptr,
				*isp_request_base;
	struct isp_response	*isp_response_ptr,
				*isp_response_base;
	kmutex_t		isp_waitq_mutex;
	struct	scsi_cmd	*isp_waitf;
	struct	scsi_cmd	*isp_waitb;
	kmutex_t		isp_pkt_alloc_mutex;
	kmutex_t		isp_pkt_free_mutex;
	int			isp_resource_cb_id;
	caddr_t			isp_std_pkt_pool_anchor;
	caddr_t			isp_std_pkt_pool_free;
	caddr_t			isp_ext_pkt_pool_anchor;
	caddr_t			isp_ext_pkt_pool_free;
#ifdef ISP_PERF
	u_int			isp_request_count,
				isp_mail_requests;
	u_int			isp_intr_count,
				isp_perf_ticks;
	u_int			isp_rpio_count,
				isp_wpio_count;
#endif
	int			isp_timeout_id;
	u_char			isp_tags[NTARGETS_WIDE];
	u_short			isp_last_tgt_watched;
	struct	isp_slot	isp_slots[ISP_MAX_SLOTS];
};

#endif /* ISP_H */
