/*
 * pln.c  -  process PLN host adapters.
 *
 * PLN support by Jim Pirzyk <pirzyk@hotmail.com>
 * 
 */

#ifndef lint
static char rcsid[] = "$Id: pln.c,v 1.5 2000/03/23 22:09:39 jdd Exp $";
#endif

#ifdef PLN_SUPPORT

#include "uscsi.h"
#include "format.h"
#include <stdio.h>

#ifdef FILENAME_MAX     /* defined in stdio.h only for SunOS >=5.x */
#define SYSV
#endif

#include <sys/param.h>
#include <values.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <nlist.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include "utils.h"
#include "common.h"
#include "param.h"

#ifdef SYSV
#include "sunos5.h"

#include <sys/fc4/fcp.h>
#include <sys/fc4/fc_transport.h>

#ifdef _NOTE
#undef _NOTE
#endif /* _NOTE */
#define _NOTE(x)

#define _KMEMUSER
#include <sys/scsi/adapters/plndef.h>
#include <sys/scsi/targets/pln_ctlr.h>
#include <sys/scsi/adapters/plnvar.h>

#ifndef CNUM
#define CNUM(pln)      (ddi_get_instance(pln->pln_tran->tran_hba_dip))
#endif /* CNUM */

static void pln_controller_message (cnum, pln) 
int cnum;
struct pln *pln;
{
	printf("pln%d:", cnum);
	printf(" type FC-PH v%2.1f,", (float) (FC_PH_VERSION - 2));
	printf(" %hu internal busses, %hu targets per bus", 
		pln->pln_nports, pln->pln_ntargets);
     printf("\n");
}

static u_int pln_get_speed(pln, i)
struct pln *pln;
int i; /* target index */
{
	return 0;
}

static void pln_synch_message(pln, i, speed)
struct pln *pln;
int i;
u_int speed;
{
	printf("Unknown");
	printf(" Clean");
	printf(" CannotReconnect");
	printf(" TaggedQueuing");
	printf(" Wide");
}

int process_pln()
{
	u_long addr;
	struct pln pln[1];
	int noisycount=0; /* number of noisy devices */
     u_int j, npln;


	if(-1==kvm(nl[N_PLN_SOFTC].n_value, addr)){
		return(0); /* pln driver not loaded */
	}
	if((u_long)NULL==addr) return(0); /* no pln's */ 

	if(-1==kvm(addr, pln[0])){
		Error("kvm read: initial pln");
	}

	npln=get_npln();
     if(0==npln) return(0);

	for(j=0;j<npln;j++){
		struct dev_info di, *next;
		char *devicepath, *dname;
		scsi_hba_tran_t tran;

		if((u_long)NULL==addr) break;

		if(-1==kvm(addr, pln[0])){
			Error("kvm read: pln #%d", j);
		}

		/*
		 * Grab the transport devinfo structure for the pln
                 * controller. Needed for CNUM.
                 */
		if(-1==kvm((u_long)pln->pln_tran, tran)){
               Error("kvm read: #%d pln tran", j);
          }
          pln->pln_tran=(scsi_hba_tran_t *)&tran;
          if(-1==kvm((u_long)pln->pln_tran->tran_hba_dip, devi)){
               Error("kvm read: #%d pln devinfo", j);
          }
		pln->pln_tran->tran_hba_dip=(dev_info_t *)&devi;

          /*
           * grab the devinfo structure for the pln controller. CNUM
           * requires it, and we need it to determine the device names.
           */
          if(-1==kvm(pln->pln_dip, devi)){
               Error("kvm read: #%d pln devinfo", j);
          }

          devicepath = get_devicepath(j);

		pln->pln_dip=(dev_info_t)&devi; /* for CNUM */ 

		if(param_is_set(P_CONTROLLER)) {
               pln_controller_message(CNUM(pln), pln);
          }

          init_scsitarget();

	  /*
           * Now, traverse pln's devinfo child list, looking for
           * attached drivers (i.e. DDI_CF2 is true). If we find
           * an attached driver, we presume it is an active device.
           */

		for(next=ddi_get_child(&devi);
			NULL!=next;
			next=ddi_get_next_sibling(&di)){
			char *devicename, *addrname;
               int target, lun;

               if(-1==kvm(next, di)){
				Error("kvm read: #%d esp devinfo child", j);
               }

               if(!DDI_CF2(&di)) continue; /* no device attached */

               devicecount++;

               /*
                * Get the device name and address, and
                * compute the device path for it
                */

               /* get device name */
               dname=s(kvmstr((u_long)ddi_get_name(&di)));
               /* ... and address */
               addrname=s(get_addr(&di));

               devicename = compute_devicename(dname,
                              devicepath, addrname);

               if(-1==get_scsiaddr(dname, &di, &target, &lun)){
                    Error("scsiaddr: #%d", j);
               }

               banner(param_is_set(P_FORMATDAT),
                    param_is_set(P_REPORT), devicecount);
               if(!param_is_set(P_QUIET)){
                    if(param_is_set(P_SUNOS4STYLE)) {
                         if(lun==0){
                              boot_synch_message("pln",
                                   CNUM(pln), target,
                                   pln_get_speed(pln,
                                        target),
                                        NARROW);
                         }
                    } else {
                         printf("pln%d: %s%s tgt %d lun %d:\n",
                              CNUM(pln), dname, addrname,
						target, lun);
                         if(param_is_set(P_SYNCH)){
                              printf("\t");
                              pln_synch_message(pln, target,
                               pln_get_speed(pln, target));
                              printf("\n");
                         }
                    }
               }
               if(param_is_set(P_PROBE)) {
                    do_scsi_probe(dname, devicename, lun);
               }
               if(param_is_set(P_REPORT)) {
                    report(param_is_set(P_FORMATDAT),
                         osrelease, devicename,
                         param_is_set(P_ORIGINAL));
               }
               free(devicename);
               free(addrname);
               free(dname);
          }

          addr=(u_long)pln->pln_next;

     }
     return(noisycount);
}

#else /* !SYSV */
/* PLN isn't supported on SunOS 4.x */
int process_pln(){}
#endif /* SYSV */
#endif /* PLN_SUPPORT */
